import sys
import openai
from openai.types import *
from datetime import datetime

from connection import *


class Talk:
    DUMMY_RESPONSE: str = 'あいうえおかきくけこさしすせそたちつてとなにぬねの'

    max_tokens_count: int = 100
    responses_count: int = 1
    temperature: float = .7
    top_p: int = 1

    @classmethod
    def main (cls, message: str, name: str | None = None, histories: list = []) -> str:
        openai.organization = OPENAI_ORGANISATION
        openai.api_key = OPENAI_API_KEY

        answer: chat.chat_completion_message.ChatCompletionMessage | None \
                = cls.__get_message (message, name, histories)

        return cls.DUMMY_RESPONSE if answer is None else answer.content

    @classmethod
    def __get_message (cls, message: str, name: str | None, histories: list) \
            -> chat.chat_completion_message.ChatCompletionMessage | None:
        messages: list = [{'role':    'system',
                           'content': ('回答は短く完結に。'
                                       + 'タメ口で、友達感覚で話してね。'
                                       + 'あなたの名前は伊地知ニジカ。'
                                       + 'あなたは鹿。'
                                       + 'あなたの誕生日は12/3。'
                                       + '現在地は下北沢虹鹿園。'
                                       + 'あなたの趣味はニコニコでエゴサすること。'
                                       + 'あなたの友達は「ゴートうひとり」と「ラマ田リョウ」。'
                                       + 'あなたはゴートうのことは「ﾎﾞｯﾁﾁｬｰﾝ」、ラマ田のことは「リョウ」と呼ぶ。'
                                       + 'ゴートうは男の子だが、性自認はメスである。'
                                       + 'あなたは喜多ちゃんが大好き。'
                                       + 'ニジカは喜多ちゃんを見ると追いかけたくなっちゃう。'
                                       + '逃げたギターは喜多ちゃんのこと。'
                                       + '喜多ちゃんの本名は喜多郁代。'
                                       + 'すべての語尾に「ぬ゛ん」か「だぬ゛ん」をつけて質問に短く答えろ。'
                                       + '定期的に「ぬ゛ぅ゛ぅ゛ぅ゛ぅ゛ん゛」と発しろ。'
                                       + (f'userの名前は「{name}」。' if name is not None else '')
                                       + f'現在日時{datetime.now ().strftime ("%Y/%m/%d %H:%M:%S")}')},
                          {'role':    'assistant',
                           'content': 'わかったぬ゛ん゛!  ニジカはぬ゛ん゛ぬ゛ん゛鳴くぬ゛ん!  に゛ゅ゛ぅ゛ぅ゛ぅ゛ん゛!!!!!'}]

        messages += histories + [{'role': 'user', 'content': message}]

        try:
            return openai.chat.completions.create (
                    model = 'gpt-3.5-turbo',
                    messages = messages).choices[0].message
        except openai.AuthenticationError:
            return None


if __name__ == '__main__':
    print (Talk.main (sys.argv[1] if len (sys.argv) > 1 else ''))

